#--------------------------------------------------
# Analyze Length-Frequency data
# Determine Lc and Lmean, plot relative to reference points
# Code written by Rainer Froese in March 2016 for use in WKIND3.3i
# Uncomment (= delete #) stocks that you want to analyze
# R code and data files have to be in the same directory
# Data file has to have column names Stock, Year, Length, CatchNo, where length is in mm
# Reference points are from SMALK_Analysis_28c.r or from the literature
#--------------------------------------------------
rm(list=ls(all=TRUE)) # clear previous variables etc
options(digits=3) # displays all numbers with three significant digits as default
graphics.off() # close graphics windows from previous sessions

# Should results be written into a file?
outfile        <- paste("LF_Out_",format(Sys.Date(),format="%B%d%Y"),".csv",sep="") # default name for output file
write.output   <- F # set to TRUE if file with Lc, Lmean, Lpeak, Lmax by year is wanted, else set to FALSE
# Because data are appended in outfile, there are no headers. Insert headers manually: Stock, Lc, Lmean, Lpeak, Lmax 

# Initialize variables
Stock         <- NA
Species       <- NA
Linf          <- NA
K             <- NA
userLopt      <- NA
userLc_opt    <- NA
M             <- NA
FM            <- NA
Species       <- NA
StartYear     <- NA
EndYear       <- NA
Comment       <- ""

# Data sources and stock designations
# -----------------------------------------
# WKIND3.3i Species
#-------------------------------------------------------

# Northern hake, hke-nrtn, growth data from EVHOE (also used in Assessment)
# Northern hake, hke-nrth, growth data from Stock Assessment: Linf=130; K estimate by the model
# File         <- "LF_ComStocks_3.csv" 
# Species      <- "Merluccius merluccius"
# Stock        <- "hke-nrtn"
# StartYear    <- 2000
# EndYear      <- 2013
# M            <- 0.4 # from advice document 
# FM           <- mean(0.31, 0.33, 0.34) # F in assessment for 2012 - 2014
# # Linf         <- 109 
# Linf         <- 130 # used in the assessment
# K            <- 0.177 # estimate by the stock assessment model (WGBIE2015)
# Lm50         <- 42.85 # Used in stock assessment (WGBIE2015)
# Lm90         <- 1.2 * Lm50 # no estimates available, assuming +20%
# Comment      <-"Linf, K, M, Lm50 from assessment; Lm90 assumed as 1.2*Lm50"

# Mediterranean hake, GSA 9,
#File           <- "hake_9_ICES_2_com_sorted.csv"
#Species        <- "Merluccius merluccius"
#Stock          <- "hke-med"
#StartYear      <- 2006
#EndYear        <- 2014
#Linf           <- mean(96.8,97.7,100.7) # based on information in FB
#K              <- mean(0.08,0.13,0.25)  # based on information in FB
#FM             <- NA
#M              <- NA
#Lm50           <- mean(c(29.0, 32.2,36.3)) # females in meds,from literature in FB
#Lm90           <- 1.2 * Lm50 # no estimates available, assuming +20%
#Comment        <- "It is unclear whether this length frequency is representative of the whole stock "

# North Sea plaice, ple-nsea, German LF data
# File         <- "LF_ComStocks_3.csv" # file is in Software directory of WKIND33
# Species      <- "Pleuronectes platessa"
# Stock        <- "ple-nsea"
# StartYear    <- 2010
# EndYear      <- 2014
# Linf         <- 49.9 # combined sexes 
# K            <- 0.153 # combined sexes
# Lm50         <- 22.8 # for females
# Lm90         <- 30.7 # for females
# M            <- 0.1 
# FM           <- 0.22 
# userLopt     <- 40.9 # combined sex
# userLc_opt   <- 36.7 # combined sex
# Comment      <- "Linf, K, Lopt and Lc_opt for combined sex; Lm50 and Lm90 for females"
 
# North Sea cod, cod-347d, German LF data
# File         <- "LF_ComStocks_3.csv" # file is in Software directory of WKIND33
# Species      <- "Gadus morhua" 
# Stock        <- "cod-347d" 
# Linf         <- 114 # combined sex 
# K            <- 0.213 # combined sex 
# Lm50         <- 53.4 # females
# Lm90         <- 79.9 # females
# M            <- 0.21 
# FM           <- 0.393 
# userLopt     <- 85.5 # combined sex
# userLcopt    <- 75.6 # combined sex
# Comment      <- "Linf, K, Lopt, Lc_opt for combined sex; Lm50 and Lm90 for females"

# Eastern Baltic cod
# File      <- "EBC_com00_14.csv"
# Species   <- "Gadus morhua" 
# Stock     <- "cod-2532"
# Linf      <- 90
# K         <- 0.3
# Lm50      <- 32.9 
# Lm90      <- 42.2 
# M         <- 0.3 
# Comment   <- "Low Linf enforced" 
 
# spurdog (dogfish) NEA, dgs-nea,
# MALES
# File        <- "spurdog_scot_MALES.csv"
# Species     <- "Squalus acanthias"
# Stock       <- "dgs_nea"
# StartYear  <- 1991
# EndYear    <- 2004
# Linf        <- 116.3 # (Fishbase)
# K           <- 0.1 # (Fishbase)
# M           <- 0.1
# # userLopt     <- 2/3*Linf
# # userLc_opt   <- 1/2*Linf
# FM          <- 0.014
# Lm50        <- 64 #(used in assessment WGEF)
# # Lm50        <- mean(74,75,77,82,83,83) # female, from Lit. in FB
# # Lm90        <- mean(93,94) # female, from Lit in FB
# Lm90        <- 1.2 *Lm50
# Comment     <- "Data for males"


# # spurdog (dogfish) NEA, dgs-nea,
# #FEMALES
# File        <- "spurdog_scot_FEMALES2.csv"
# Species     <- "Squalus acanthias"
# Stock       <- "dgs-nea"
# StartYear  <- 1991
# EndYear    <- 2004
# Linf        <- 116.3 # (Fishbase)
# K           <- 0.1 # (Fishbase)
# M           <- 0.1
# userLopt     <- 2/3*Linf
# userLc_opt   <- 1/2*Linf
# FM          <- 0.014
# Lm50        <- 80 #(used in assessment WGEF)
# # Lm50        <- mean(74,75,77,82,83,83) # female, from Lit. in FB
# # Lm90        <- mean(93,94) # female, from Lit in FB
# Lm90        <- 1.2 *Lm50
# Comment     <- "Data for females; Lm50 from assesment"

# Roundnose Grenadier
 File          <- "LF_ComStocks_rng_RNG_FR-Lendist.csv"
 Species       <- "Coyphaenoides rupestris"
 Stock         <- "rng-5b67"
 # Linf          <-  23.9 # PAFL based on Lit in FB
 Linf          <- 27 # based on max lengths in data
 K             <- 0.06  # based on Lit in FB
 FM            <- mean(0.462*0.08, 0.687*0.08, 0.392*0.08) # Harvest rate / Fmsy = 0.08 for 2012-2013
 Lm50          <- 2.29+0.196*mean(46.5, 54.5) # for females, based on Lit in FB
 Lm90          <- 1.2* Lm50 # assumed 20% higher than Lm50
 Comment       <- "Length type is preanal fin length, with PAFL = 0.196*TL+2.29; ref points were adjusted"

# Mediterranean anchovy, GSA 17-18, Engraulis encrasicolus 
# File           <- "Anchovy_1718_ICES_RF.csv"
# Species        <- "Engraulis encrasicolus"
# Stock          <- "anc-GSA1718"
# Linf           <- 20 # based on Lit in FB
# K              <- 0.4 # based on Lit in FB
# FM             <- NA
# M              <- NA
# Lm50           <- mean(10.5, 11.5, 11.2) # based on female and unsexed data in Lit in FB
# Lm90           <- mean(14, 13.5) # based on female and unsexed data in Lit in FB
# Comment        <- "Growth and maturity from literature"

# Giant red shrimp, GSA 11
# File            <- "Gian_Red_shrimp_11_ICES_RF.csv"
# Species         <- "Aristaeomorpha foliacea"
# Stock           <- "GRShrimp11"
# Linf            <- 7 # from Lit in SeaLifeBase
# K               <- 0.45
# FM              <- NA
# M               <- NA
# Lm50            <- mean(3.05, 3.9, 3.71,4.2 ) # from Lit in SeaLifeBase
# Lm90            <- 1.2 * Lm50 # assumed 20% higher
# Comment         <- "Carapace length"

# Atlantic swordfish, Xiphias gladius, 
# File             <- "SWO_AS.csv"
# Species          <- "Xiphias gladius"
# Stock            <- "SWO_AS"
# StartYear        <- 1975 
# EndYear          <- 2003
# Linf             <- mean(264,267,277) # FL North Atlantic, Lit in FB # 238.6 of ICCAT is below data
# K                <- mean(0.12,0.12,0.07) # 0.185 of ICCAT
# Lm50             <- 156 # ICCAT for females
# Lm90             <- 1.2 * Lm50 # assumed + 20%
# M                <- 0.2 # ICCAT
# Comment          <- "M and Lm50 from assessment; Linf from lit."


#--------------------------------------
# read data file
#--------------------------------------
data.raw <- read.csv(File, header=T, stringsAsFactors=F) 

# restrict data to stock
data.raw <- data.raw[data.raw$Stock == Stock,] 
 
# use StartYear and EndYear if given
if(is.na(StartYear)==F & is.na(EndYear)==F) {
   data.raw <- data.raw[data.raw$Year >= StartYear &
                        data.raw$Year <= EndYear,] }

# Simplify data
AllYear    <- data.raw$Year
AllLength  <- data.raw$Length/10 # change from mm to cm
AllFreq    <- data.raw$CatchNo/1000 # reduce numbers for better display
Years      <- sort(unique(AllYear))
nYears     <- length(Years)
# Create matrix to store annual Lc and Lmean
Ldat      <- data.frame(Stock=rep(Stock,nYears),Year=rep(NA,nYears),
                         Lc=rep(NA,nYears), Lmean=rep(NA,nYears),
                         Lpeak=rep(NA,nYears), Lmax=rep(NA,nYears))

#--------------------------------------------
# Calculate reference points Lopt and Lc_opt
#--------------------------------------------

LoptLinf      <-  2/3*Linf 

if(is.na(M)==F & is.na(K)==F) {
  LoptMK      <-  Linf*3/(3+M/K)} else LoptMK <- NA
  
if(is.na(M)==F & is.na(K)==F & is.na(FM)==F) {
  Lc_optMKF   <-  Linf*(2+3*FM/M)/((1+FM/M)*(3+M/K)) } else Lc_optMKF <- NA

Lc_optF1M   <- 0.5*Linf
Lc_optF2M   <- 0.55*Linf 

#-----------------------------------------
# Start LF analysis by year
#-----------------------------------------
i = 0 # start counter
for(Year in Years) {
  i = i+1
  # get data for the specific year
  Length  <- AllLength[AllYear==Year]
  Freq    <- AllFreq[AllYear==Year]
  
  # enter data for this year into data frame
  Ldat$Year[i]     <- Year
  Ldat$Lmax[i]     <- max(Length[Freq>0])
  Ldat$Lmean[i]    <- sum(Length*Freq)/sum(Freq)
  
  # get most frequent length as mean of 3 lengths with highest Freq
  Freq.sort         <- sort(Freq)
  Freq.max3         <- Freq.sort[(length(Freq.sort)-2)] 
  Ldat$Lpeak[i]    <- mean(Length[which(Freq>=Freq.max3)])

  # get length where 50% of the individuals are retained by the gear as half of mean max Freq
  Freq.max3.mean    <- mean(Freq.sort[(length(Freq.sort)-2):length(Freq.sort)]) 
  # use mean of lengths with freqs between 20% and 80% of max freq
  Ldat$Lc[i]        <- mean(Length[which(Length<Ldat$Lpeak[i] &
                                           Freq>0.2*Freq.max3.mean &
                                           Freq<0.8*Freq.max3.mean)])   
 
} # end of LF analysis by year loop 

#---------------------------------------------------------
# Determine values of Lopt and Lc_opt to be used in graphs
#---------------------------------------------------------
Lopt    <- ifelse(is.na(userLopt)==F,userLopt,ifelse(is.na(LoptMK)==F,LoptMK,LoptLinf))
Lc_opt  <- ifelse(is.na(userLc_opt)==F,userLc_opt,ifelse(is.na(Lc_optMKF)==F,Lc_optMKF,Lc_optF1M))

#------------------------------------------
# plot histogram for first year
#------------------------------------------
windows(12,6)
par(mfrow=c(1,2))
LengthFirst <- AllLength[AllYear==min(Years)]
FreqFirst   <- AllFreq[AllYear==min(Years)]
YearFirst   <- Years[1]
LmeanFirst  <- Ldat$Lmean[1]
LcFirst     <- Ldat$Lc[1]
FrmaxFirst  <- max(FreqFirst)


plot(x=LengthFirst,y=FreqFirst, type="h", 
     xlim=c(0,max(c(1.1*Linf,max(LengthFirst)))), bty="l",
     ylim=c(0,1.1*FrmaxFirst),
     xlab="Length (cm)", ylab="Frequency", main=paste(Stock,YearFirst))
lines(x=c(LmeanFirst,LmeanFirst), y=c(0,FrmaxFirst), col="red", lwd=2) # Lmean
lines(x=c(LcFirst,LcFirst), y=c(0,0.5*FrmaxFirst), col="red", lwd=2) # Lc
lines(x=c(Lm50,Lm50), y=c(0,FrmaxFirst), col="green")
text(x=Lm50,y=FrmaxFirst, "Lm50", col="green", adj=c(0.5,-0.5))
lines(x=c(Lopt,Lopt), y=c(0,FrmaxFirst), col="green")
text(x=Lopt,y=FrmaxFirst, "Lopt", col="green", adj=c(0.5,-0.5))
lines(x=c(Linf,Linf), y=c(0,FrmaxFirst), col="green")
text(x=Linf,y=FrmaxFirst, "Linf", col="green", adj=c(0.5,-0.5))


#------------------------------------------
# plot histogram for last year
#------------------------------------------
LengthLast <- AllLength[AllYear==max(Years)]
FreqLast   <- AllFreq[AllYear==max(Years)]
YearLast   <- Years[nYears]
LmeanLast  <- Ldat$Lmean[nYears]
LcLast     <- Ldat$Lc[nYears]
FrmaxLast  <- max(FreqLast)

plot(x=LengthLast,y=FreqLast, type="h", 
     xlim=c(0,max(c(1.1*Linf,max(LengthLast)))), bty="l",
     ylim=c(0,1.1*FrmaxLast),
     xlab="Length (cm)", ylab="Frequency", main=paste(Stock,YearLast))
lines(x=c(LmeanLast,LmeanLast), y=c(0,FrmaxLast), col="red", lwd=2) # Lmean 
lines(x=c(LcLast,LcLast), y=c(0,0.5*FrmaxLast), col="red", lwd=2) # Lc
lines(x=c(Lm50,Lm50), y=c(0,FrmaxLast), col="green")
text(x=Lm50,y=FrmaxLast, "Lm50", col="green", adj=c(0.5,-0.5))
lines(x=c(Lopt,Lopt), y=c(0,FrmaxLast), col="green")
text(x=Lopt,y=FrmaxLast, "Lopt", col="green", adj=c(0.5,-0.5))
lines(x=c(Linf,Linf), y=c(0,FrmaxLast), col="green")
text(x=Linf,y=FrmaxLast, "Linf", col="green", adj=c(0.5,-0.5)) 
  
#----------------------------------------------
#  Plot time series of Lc and Lmean
#----------------------------------------------
windows(8,6)  
rightspace <- ifelse(nYears < 10,1,3) # space to display labels
plot(x=Ldat$Year,y=Ldat$Lmean, bty="l",type = "l", 
     xlim=c(Years[1],Years[nYears]+rightspace), 
     ylim=c(0,max(1.1*Lopt,max(Ldat$Lmean),1.1*Lm90)),lwd=2,
     xlab="Year",ylab = "Lenght (cm)",main=Stock, xaxt="n")
axis(1, at=Ldat$Year,labels=sprintf("%1.0f",Ldat$Year)) 
text(x=Years[nYears],y=Ldat$Lmean[nYears],"Lmean", adj=c(-0.2,0))
lines(x=Ldat$Year,y=Ldat$Lc,lwd=1.5) # line for Lc
text(x=Years[nYears],y=Ldat$Lc[nYears],"Lc", adj=c(-0.2,0))
lines(x=Ldat$Year,y=rep(Lopt,nYears),lty="dashed", col="green") # line for Lopt
text(x=Years[nYears],y=Lopt,"Lopt", adj=c(-0.2,0), col="green")
lines(x=Ldat$Year,y=rep(Lm90,nYears),lty="dashed", col="green") # line for Lm90
text(x=Years[nYears],y=Lm90,"Lm90", adj=c(-0.2,0), col="green")
lines(x=Ldat$Year,y=rep(Lm50,nYears),lty="dashed", col="green") # line for Lm50
text(x=Years[nYears],y=Lm50,"Lm50", adj=c(-0.2,0), col="green")
lines(x=Ldat$Year,y=rep(Lc_opt,nYears),lty="dashed", col="green") # line for Lc_opt
text(x=Years[nYears],y=Lc_opt,"Lc_opt", adj=c(-0.2,0), col="green")   


 #---------------------------------------------
 # Show results on screen
 #---------------------------------------------
  cat("---------------------------------------------\n")
  cat("Results of LFCOM analysis,",date()," \n")
  cat("Species =", Species,", stock =", Stock,"\n\n")
  cat("External estimates of Linf, K, Lm50, Lm90, M, F \n")
  cat("---------------------------------------------\n")
  cat("Asymptotic length              Linf =", Linf, "cm \n")
  cat("Growth parameter                  K =", K, "1/year \n")
  cat("Length at 50% maturity         Lm50 =", Lm50, "cm  \n")
  cat("Length at 90% maturity         Lm90 =", Lm90, "cm  \n")
  cat("Natural mortality of adults       M =", M, "1/y \n")
  cat("Commercial fishing mortality      F =", FM, "1/y \n")
  cat("M/K (expected 1.0-2.0)          M/K =", M/K, "\n")
  cat("Comment:", Comment, "\n\n")  
  
if(is.na(M)==F & is.na(K)==F & is.na(FM)==F) {
  cat("----------------------------------------------\n")
  cat("Lopt, Lc_opt and L(F=M) based on Linf, M, K, F\n")
  cat("----------------------------------------------\n")
  cat("Length at max cohort biomass Lopt   =", LoptMK, "\n") 
  cat("Lc resulting in Lopt         Lc_opt =", Lc_optMKF, "\n")
  cat("Mean length in catch if F=M  L(F=M) =", (K*Linf+2*LcLast*M)/(K+2*M),"cm \n\n")
  } 
  cat("----------------------------------------------\n")
  cat("Lopt, Lc_opt and L(F=M) based on Linf\n")
  cat("----------------------------------------------\n")
  cat("Length at max cohort biomass Lopt   =", LoptLinf,"cm, assuming b ~ 3 and M/K ~ 1.5 \n")
  cat("Lc resulting in Lopt         Lc_opt =", Lc_optF1M,"cm, if F ~ M \n")
  cat("Lc resulting in Lopt         Lc_opt =", Lc_optF2M,"cm, if F ~ 2 M \n")
  cat("Mean length in catch if F=M  L(F=M) =", 0.75*LcLast+0.25*Linf,"cm\n\n")

  
  cat("---------------------------------------------\n")
  cat("Observed Lc and Lmean in",YearLast,"       \n")
  cat("---------------------------------------------\n")
  cat("Mean length at first capture   Lc =", LcLast,"cm \n")
  cat("Mean length in catch        Lmean =", LmeanLast,"cm \n\n")
 
  
  cat("Time series \n")
  cat("---------------------------------------------\n")
  print(Ldat)
  
#--------------------------------------------
# Write some results into csv outfile
#--------------------------------------------
    if(write.output == TRUE) {
    write.table(Ldat, file=outfile, append = T, sep = ",", 
                dec = ".", row.names = FALSE, col.names = FALSE) }
  
  