#--------------------------------------------------
# Analyze SMALK data from DATRAS.ICES.dk
# Determine LWR, Growth and Maturity
# Code originally written by Rainer Froese in 2013
# Updated for use in WKIND3.3i in 2016
# Uncomment (= delete #) stocks that you want to analyze
# R code and data files have to be in the same directory
# Install package "weights"
#--------------------------------------------------
rm(list=ls(all=TRUE)) # clear previous variables etc
require(weights) # for wtd.hist(), which is used to get weighted frequencies for ogives
options(digits=3) # displays all numbers with three significant digits as default
graphics.off() # close graphics windows from previous sessions

# Initialize variables; change in stock designations below, if needed
UserLinf 	  	<- NA
Usert0        <- NA
Usertmax      <- NA
Linf          <- NA
K             <- NA
M             <- NA
FM            <- NA
Area          <- "All"
FB_Lmax       <- 10000
Species       <- NA
StartYear     <- NA
EndYear       <- NA
Quarter       <- NA
m.okay        <- FALSE
UseMisWeight  <- FALSE # include records without weight (default = FALSE)
Sex           <- "F"  # (set "F" for female, "M" for male or NA for mixed; default is "F") 
SkipVBGF      <- FALSE # set to TRUE if VBGF does not converge
UseQuarters   <- FALSE # set to TRUE if more than 1 quarter is used to account for ageing with the year; assumes birth in first quarter 
Comment       <- ""

# Data sources and stock designations
# -----------------------------------------
# WKIND3.3i Species
#-------------------------------------------------------
# North Sea, Datras, NS-IBTS, Q1, from http://datras.ices.dk 
#-------------------------------------------------------

# North Sea herring, her-47d3
# source for M is page 126 in http://www.ices.dk/sites/pub/Publication%20Reports/Expert%20Group%20Report/acom/2015/HAWG/01%20HAWG%20Report%202015.pdf
# File      <- "SMALK_her-nsea_NS-IBTS_-2016_3Q.csv"
# Species   <- "Clupea harengus" 
# Area      <- c(1,2,3,4,5,6,7)
# StarYear  <- 1990
# Quarter   <- 3 
# M         <- mean(0.38, 0.35, 0.34, 0.33, 0.32) # M at age 0-8 in 2012: 0.96 0.66 0.38 0.35 0.34 0.33 0.32 0.31 0.31 
# FM        <- mean(0.159, 0.194, 0.203) # mean of F2-6 for 2012-2014, from 2015 advice document

# North Sea cod, cod-347d
 File       <- "SMALK_NS-IBTS_2016-02-19_Q1.csv"
 Species    <- "Gadus morhua" 
 Area       <- c(1,2,3,4,5,6,7)
 StartYear  <- 2000
 Quarter    <- 1
 M          <- 0.21  
 FM         <- mean(0.393, 0.385,0.393) # Fbar2-4 for 2012 - 2014 

# North Sea plaice, ple-nsea
# File       <- "SMALK_NS-IBTS_2016-02-19_Q1.csv"
# Species    <- "Pleuronectes platessa"  
# Area       <- c(1,2,3,4,5,6,7)
# StarYear   <- 2000
# Quarter    <- 1
# UserLinf   <- 55
# M          <- 0.1 # from stock assessment graphs db
# FM         <- mean(0.22, 0.18, 0.18) # F2-6 for 2012 - 2014

# North Sea sandeel, san-ns1-5 
# File         <- "SMALK_Ammodytes_NS-IBTS_2016-02-19.csv"
# Species      <- c("Ammodytes marinus","Ammodytes")  # %in%("Ammodytes","Ammodytes marinus","Ammodytes tobianus"
# Area         <- "All" 
# Usert0      <- -0.5
# UseMisWeight <- TRUE
# Sex          <- NA
# M            <- 0.5 # assumed, check full assessment
# FM           <- 0.5 # for san-1 
# Comment      <- "Lumped data across all stocks"
 
# Spurdog, dgs-nea [from EXCHANGE CA across all surveys, length standardized to mm in new column LngtClasMM]
#                  [also NoAtALK has to be renamed in NoAtalk and IndWgt to IndividualWeight]   
# File         <- "SqualusAcanthias_CA.csv "
# Species      <- NA # "Squalus acanthias", no Species field in EXCHANGE
# UseMisWeight <- TRUE
# Sex          <- NA
# M            <- 0.07 # from M = 1.5 K and K = 0.05 Sosinski 1977   
# FM           <- 0.014  # u for 2012-2014
# Comment      <- "Squalus acanthias across all surveys and areas"
 
# Northern hake, hke-nrtn, growth data from EVHOE (also used in Assessment)
# File         <- "SMALK_Hake_EVHOE_2016-02-23.csv" 
# Species      <- "Merluccius merluccius"
# Sex          <- NA
# UserLinf     <- 109
# M            <- 0.4 # from advice document 
# FM           <- mean(0.31, 0.33, 0.34) # 2012 - 2014
# Comment      <- "No weight and no maturity data"

# Northern hake, hke-nrtn, EXCHANGE CA data, survey IE-IGFS, all years and Q, but only LWR works, age reading is wrong, no maturity
# The following fields were renamed: LngtClas -> LngtClasMM, IndWgt -> IndividualWeight, NoAtALK -> NoAtalk
# File         <- "Exchange_CA_IE-IGFS_Hake.csv" 
# Species      <- NA # "Merluccius merluccius", SpecCode W = 126484
# StartYear    <- 2000
# SkipVBGF     <- TRUE
# UserLinf     <- 109
# Sex          <- "F"
# M            <- 0.4 # from advice document 
# FM           <- mean(0.31, 0.33, 0.34) # 2012 - 2014
# Comment      <- "Age data unreliable; no maturity data"
 
# ------------------------------------------------------
# Baltic, BITS
#-------------------------------------------------------
# Western Baltic cod, cod-2224
# File  <- "SMALK_Cod_BTS_2016-02-23.csv"
# Species   <- "Gadus morhua"
# Quarter   <- 1
# Area      <- c(22,24)
# StartYear <- 2000
# UserLinf <- 119
# M         <- 0.2
# FM        <- mean(0.88, 0.94, 0.84)    # F3-5 for 2012 - 2014

# Eastern Baltic cod, cod-2532
# File  <- "SMALK_Cod_BTS_2016-02-23.csv"
# Species   <- "Gadus morhua"
# Quarter   <- 1
# Area      <- c(25,26,27,28,29,30,31,32)
# StartYear <- 2000
# UserLinf  <- 119
# M         <- 0.2

 
#--------------------------------------
# read data file
#--------------------------------------
data.raw <- read.csv(File, header=T, stringsAsFactors=F)

# restrict data to selected area
inArea <- function(x, table) {match(x, table, nomatch=0) >0}
if(Area[1] != "All") {
 data.raw <- data.raw[inArea(data.raw$Area,Area),] 
} 

# use StartYear and EndYear if given
if(is.na(StartYear)==F) {
  data.raw <- data.raw[data.raw$Year >= StartYear,] 
}
if(is.na(EndYear)==F) {
  data.raw <- data.raw[data.raw$Year <= EndYear,] 
}

# restrict data to species
if(is.na(Species[1])==F) {
  data.raw <- data.raw[data.raw$Species %in%(Species),]
}
 
# restrict data to selected Sex
if(is.na(Sex) == FALSE) {
  data.raw <- data.raw[data.raw$Sex==Sex,]
}

# restrict data to Quarter
if(is.na(Quarter) == F) {
    data.raw <- data.raw[data.raw$Quarter==Quarter,]
  }    


# since length and N are needed for all applications, exclude records where either is NA or zero
# or length is larger than the maximum length recorded in FishBase (FB_Lmax)
# or age is larger than Usertmax
data.raw <- data.raw[is.na(data.raw$LngtClasMM)==F & data.raw$LngtClasMM > 0,] 
data.raw <- data.raw[is.na(data.raw$NoAtalk)==F & data.raw$NoAtalk > 0,] 
data.raw <- data.raw[data.raw$LngtClasMM/10 < FB_Lmax,]

# exclude unlikely maximum ages
if(is.na(Usertmax) == F) {
data.raw <- data.raw[data.raw$Age < Usertmax,]
}

# bind vectors into data frame
data.org <- data.frame(Year=as.numeric(data.raw$Year),Quarter=as.numeric(data.raw$Quarter),
                       Length=as.numeric(data.raw$LngtClasMM)/10, Weight=as.numeric(data.raw$IndividualWeight), 
                       N=as.integer(data.raw$NoAtalk), Age=as.numeric(data.raw$Age), Sex=as.character(data.raw$Sex), 
                       Mat=data.raw$Maturity, stringsAsFactors=F)

cat("\n")
cat("----------------------------------------------","\n")
cat("Results of SMALK analysis,", date(), "\n")
cat("----------------------------------------------","\n")
cat("File    =", File, "\n")
cat("Survey  =", unique(data.raw$Survey), "\n")
cat("Species =", as.character(unique(data.raw$Species)), "\n")
if(is.na(Sex)==F) cat("Sex     =", Sex, "\n") else cat("Sex     =", unique(data.raw$Sex), "\n")
cat("Years   =", min(data.org$Year), " -", max(data.org$Year), "\n") 
if(is.na(Quarter)==F) cat("Quarter =", Quarter, "\n") else cat("Quarter =", unique(data.raw$Quarter), "\n")
cat("Areas   =", unique(data.raw$Area), "\n")

# do LWR if enough data points are available
if(length(data.org$Weight[is.na(data.org$Weight)==F & data.org$Weight!=-9 ]) >= 10) {
  LWR.okay <- TRUE } else { LWR.okay <- FALSE}
if(LWR.okay==TRUE) {
  #-------------------------------------------------------------
  # do preliminary LWR analysis to detect and remove extreme outliers
  #--------------------------------------------------------------
  # For LWR analysis, clean weight data from missing values and erroneous zeros
  data.lw <- data.org[is.na(data.org$Weight)==F  & data.org$Weight > 0,] 

  model <- lm(log10(data.lw$Weight) ~ log10(data.lw$Length), weights = data.lw$N) 
  residuals   <- resid(model)
  SD.residuals <- sd(residuals)
  outliers	<- which(abs(residuals) > 4 * SD.residuals)
  if(length(outliers) > 0) { data.lw	<- data.lw[-outliers,] } # remove outliers if present

  #------------------------------------------------------------
  # plot dirty data with 4 SD 
  #------------------------------------------------------------
  windows(12,6)
  par(mfrow=c(1,2))
  plot(log10(data.org$Length),log10(data.org$Weight), 
     xlab=expression("log"[10]*"("~italic(L)~")"), 
     ylab=expression("log"[10]*"("~italic(W)~")"), 
     main="LWR data with 4*SD lines and outliers", las=1, frame =T)
  abline(model)
  abline(a=summary(model)$coefficients[1,1] + 4 * SD.residuals, b=summary(model)$coefficients[2,1], lty=3)
  abline(a=summary(model)$coefficients[1,1] - 4 * SD.residuals, b=summary(model)$coefficients[2,1], lty=3)

  # determine cleaned vectors for use in LWR, Wetherall and Maturity
  Y    <- data.lw$Year
  L    <- data.lw$Length
  log10L <- log10(L)
  Lmin <- min(L)
  Lmax <- max(L)
  W    <- data.lw$Weight
  Wmin <- min(W) 
  Wmax <- max(W) 
  log10W <- log10(W)	    
  Mat  <- data.lw$Mat
  N    <- data.lw$N # number of individuals

  # Do a weighted length-weight relationship
  model.clean	<- lm(log10W ~ log10L, weights = N)
  log10a 	<- summary(model.clean)$coefficients[1,1]
  SE_log10a <- summary(model.clean)$coefficients[1,2] 
  confint_log10a <- confint(model.clean)[1,]
  b		<- summary(model.clean)$coefficients[2,1]
  SE_b	<- summary(model.clean)$coefficients[2,2]
  confint_b <- confint(model.clean)[2,]
  r2		<- summary(model.clean)$r.squared
  SD_estimate <- sd(resid(model)) # standard deviation of the residuals and of the estimate
  meanL	<- sum(L*N) / sum(N)
  # Bias correction factor from geometric mean W to mean W
  syx       <- summary(model.clean)$sigma
  cf.clean  <- exp((syx^2/2))  
  predW   <- 10^(log10a + b * log10(meanL)) * cf.clean
  LLW  	<- 10^(log10a + b * log10(meanL) - 1.96 * SD_estimate) * cf.clean
  ULW		<- 10^(log10a + b * log10(meanL) + 1.96 * SD_estimate) * cf.clean
     

  cat("----------------------------------------------","\n")
  cat("Summary stats of weighted mixed W~L regression", "\n")
  cat("----------------------------------------------","\n")
  cat(length(outliers), " outliers (beyond 4 SD) were removed. \n")
  cat("Number of remaining observations =", sum(N),"\n")
  cat("Length range =", Lmin, " -", Lmax, "cm \n")
  cat("Weight range =", Wmin, " -", Wmax, "g \n")
  cat("log10(a) =", log10a, ", SE =", SE_log10a, "\n") 
  cat("Geometric mean a =", 10^(log10a), ", 95% CL =", 
	  10^(confint_log10a[1]), "-", 
	  10^(confint_log10a[2]), "\n")
  cat("b =", b, ", 95% CL =", confint_b[1],
	  "-", confint_b[2], "\n")
  cat("Standard deviation of estimated log10(W) =", SD_estimate, "\n")
  cat("Coefficient of determination (r2) =", r2, "\n")
  cat("Mean length =", meanL, "cm, predicted weight =", predW,
		"g (", LLW, " -", ULW, ") g \n") 

  # --------------------------------------------
  # plot data and regression
  #---------------------------------------------
  plot(x=log10L, y=log10W, 
		las=1, xlab=expression("log"[10]*"("~italic(L)~")"), 
		ylab=expression("log"[10]*"("~italic(W)~")"), 
		main="LWR with 95% range", frame =T)
  model.clean <- lm(log10W ~ log10L, weights = N)
  abline(model.clean)
  abline(a=log10a + 1.96 * SD_estimate, b=b, lty=3)
  abline(a=log10a - 1.96 * SD_estimate, b=b, lty=3)

} # end of loop where LWR.okay==TRUE

# --------------------------------------------------------------
# determine vectors for use in Wetherall and VBGF if no LWR was done
# Include records without weight if required, else use only data with observed weight
if(UseMisWeight==T | LWR.okay == FALSE) {
 Y    <- data.org$Year
 L    <- data.org$Length
 log10L <- log10(L)
 Lmin <- min(L)
 Lmax <- max(L)
 N    <- data.org$N # number of individuals
}

#--------------------------------------------------------------
# Lenght-Frequency Analysis
#--------------------------------------------------------------
# Do a weighted length frequency histogram
lf.breaks     <- 1 : (max(L)+3)
Lfreq         <- wtd.hist(L, breaks=lf.breaks, weight=N, plot=F)$counts # use counts in weighted histogram
Lpeak         <- which.max(Lfreq) 

# determine Offset = correction to Lpeak for incomplete capture of length class
r2           <- 0 # initialize variables
Offset       <- 0 # initialize variables
for(i in 1:(round((Lmax-Lpeak)/2))) {
  cut.lengths  <- seq(from=Lpeak+i, to=Lmax)
  if(length(which(Lfreq[cut.lengths] > 0)) < 3) {break} # exit loop if fewer than two length classes are occupied
  model.i      <- lm(Lfreq[cut.lengths] ~ log10(cut.lengths+0.5), weights=Lfreq[cut.lengths]) 
  if(summary(model.i)$r.squared > 1.02 * r2) { # pick first r2 that is 2% larger than the previous
    r2 <- summary(model.i)$r.squared
    Offset <- i
  } 
}

#---------------------------------------------------------------
# Get some other properties of the length-frequency distribution
#---------------------------------------------------------------
# get mid length of bins
lf.mean.lengths   <- (0 : (max(L)+1))+0.5 # first bin is 0-1 cm, mean length 0.5 cm

# get weighted mean length
Lmean             <- sum(lf.mean.lengths*Lfreq)/sum(Lfreq)

# get length at full selection by the gear
if(Offset/Lpeak < 0.1 ) {Lv <- Lpeak + Offset} else {Lv <- Lpeak + 1}

# get length where 50% of the individuals are retained by the gear as half of Max freq
Lc     <- which(Lfreq>0.5*max(Lfreq))[1]   

# Start Wetherall analysis
cut.lengths   <- seq(from=Lv, to=Lmax) # vector with cut-off lengths after the peak

mean.lengths <- c()	# create an empty vector for mean lengths
sum.N		<- c()	# create an empty vector for sum of N

# for each cut-off length, get mean length and sum of individuals
for(i in 1 : length(cut.lengths)) {
  mean.lengths[i] <- if(Lfreq[cut.lengths[i]]==0) {NA} 
  else{wtd.mean(cut.lengths[i] : Lmax, weight = Lfreq[cut.lengths[i] : Lmax])}
  sum.N[i]	<- sum(Lfreq[cut.lengths[i] : Lmax])		
}

# fit a linear regression to mean length predicted from cut-off length
model.Linf	<- lm(mean.lengths ~ cut.lengths, weights=sum.N)	
alpha.Linf 	<- model.Linf$coefficients[1]
beta.Linf	<- model.Linf$coefficients[2]
# SE.beta.Linf <- summary(model.Linf)$coefficients[2,2]
# df.Linf		<- summary(model.Linf)$df[2]
# SD.beta.Linf <- SE.beta.Linf * sqrt(df.Linf+2)

if(beta.Linf >= 1 | alpha.Linf/(1-beta.Linf) > 1.2*Lmax) {
  WA.Linf <- NA
} else {
  WA.Linf	<- alpha.Linf/(1-beta.Linf)
}
Linf <- as.numeric(WA.Linf)	# Wetherall Linf
if(is.na(WA.Linf)==T) {Linf = median(by(data.org$Length, data.org$Year, max))} # use median of max lengths in different years
if(is.na(UserLinf)==F) {Linf = UserLinf} # use user-defined Linf if present

# plot LF histogram and Wetherall analysis
windows(12,10)
par(mfrow=c(2,2))
# plot histogram
wtd.hist(L, breaks=lf.breaks, weight=N, plot=T, main="Weighted length-frequency",
         xlab="Length (cm)", ylab="Frequency")
# show Lc, Lmean, Lv, Linf 
abline(v=Lc, col="red")
text(x=Lc,y=max(Lfreq), "Lc", col="red", adj=c(1.2,0.5))
points(x=Lmean, y=Lfreq[trunc(Lmean)], col="red", cex=1) 
abline(v=Lv+0.5, col="red")
text(x=Lv+0.5, y=max(Lfreq), "Lv", col="red", adj=c(-0.5,0.5))
abline(v=Linf, col="red")
text(x=Linf,y=max(Lfreq), "Linf", col="red", adj=c(-0.2,0.5))


# plot Wetherall
plot(x=cut.lengths, y=mean.lengths, main="Wetherall weighted regression", 
     xlab="Cut-off length (cm)", ylab="Weighted mean length (cm)")
abline(model.Linf)
abline(a=0, b=1, lty=2, col="red") # 1:1 line, cross with regression = Linf
text(x=min(mean.lengths, na.rm=T), y=min(mean.lengths, na.rm=T), "1 : 1", col="red", adj=c(-2,-1))
text(x=WA.Linf, y=WA.Linf, "Linf", adj=c(0,1.3))

# determine median maximum length across years, including records without age  
median.Lmax <- median(by(data.org$Length, data.org$Year, max)) 

cat("-------------------------------------------------\n")
cat("Wetherall estimation of Linf \n")
cat("-------------------------------------------------\n")
cat("Records used =", length(L), "\n")
cat("Observed maximum length =", Lmax, "cm \n")
cat("Median of annual maximum lengths =", median.Lmax, "cm \n")
cat("Proposed Linf =", WA.Linf, "cm \n")

if(SkipVBGF==FALSE) {
#------------------------------------------------------------------------------
# Fit Von Bertalanffy Growth function (VBGF)
#------------------------------------------------------------------------------
# clean length-at-age data, # remove NA and zero age
#------------------------------------------------------------------------------
if(length(data.org$Age[data.org$Age>0]) > 5) {
if(UseMisWeight==F & LWR.okay==T) {
  data.age.pr <- data.lw[is.na(data.lw$Age)==F & data.lw$Age > 0,] } else { 
  data.age.pr <- data.org[is.na(data.org$Age)==F & data.org$Age > 0,] }

# account for aging within year (needed if more than one quarter is included)
if(UseQuarters==T) {
  data.age.pr$Age[data.age.pr$Quarter==2] <-  data.age.pr$Age[data.age.pr$Quarter==2]+0.25
  data.age.pr$Age[data.age.pr$Quarter==3] <-  data.age.pr$Age[data.age.pr$Quarter==3]+0.5
  data.age.pr$Age[data.age.pr$Quarter==4] <-  data.age.pr$Age[data.age.pr$Quarter==4]+0.75
}
   
# fit a preliminary von Bertalanffy growth curve, with Linf from Wetherall, or as median, or from user 
model.vb <- nls(data.age.pr$Length ~ Linf*(1-exp(-Kpr*(data.age.pr$Age-t0pr))), data.age.pr, 
		start=list(Kpr=0.1, t0pr=0), weights=data.age.pr$N, lower=list(Kpr=0.01, t0pr=-3),
		upper=list(Kpr=2, t0pr=1), algorithm="port") 

# preliminary estimates of K and t0 
Kpr     <- summary(model.vb)$parameters[1]
t0pr    <- summary(model.vb)$parameters[2]

# plot length at age data with curve
plot(data.age.pr$Age, data.age.pr$Length, main="Length-at-age data with outliers", xlim=c(0,max(data.age.pr$Age)),
     ylim=c(0,max(1.1*Linf,max(data.age.pr$Length))), xlab="Age (years)", ylab="Length (cm)", pch=19, cex=0.3)
x=seq(0, max(data.age.pr$Age), by=0.1)
y= Linf*(1-exp(-Kpr*(x-t0pr)))
lines(x,y)
# remove extreme outliers beyond 3 SD
residuals.vb   <- resid(model.vb)
SD.residuals.vb <- sd(residuals.vb)
outliers.vb	<- which(abs(residuals.vb) > 3 * SD.residuals.vb)
if(length(outliers.vb) > 0) { # remove outliers if present
  data.age	<- data.age.pr[-outliers.vb,] } else {  
    data.age <- data.age.pr }

#-------------------------------------------------------------------
# fit VBGF to cleaned data, plot data
#-------------------------------------------------------------------

# do second regression, 
# log-transformed model is used to force multiplicative error term with lower SD in residuals
logL        <- log(data.age$Length)
model.vb    <- nls(logL ~ log(Linf*(1-exp(-K*(data.age$Age-t0)))), data.age, 
                   start=list(K=0.1, t0=0), weights=data.age$N, algorithm="port") 
K           <- summary(model.vb)$parameters[1]
SE.K        <- summary(model.vb)$coefficients[1,2]
t0          <- summary(model.vb)$parameters[2]
SE.t0       <- summary(model.vb)$coefficients[2,2]
SD.log.residuals.vb <- sd(resid(model.vb))

# do third regression if t0 is unrealistic
# set t0 to user value or limit t0 to -3 and + 0.5 years
if(t0 < -3 | t0 > 0.5 | is.na(Usert0)==F) {
  t0set       <- ifelse(is.na(Usert0)==F,Usert0,-3)
  model.vb    <- nls(logL ~ log(Linf*(1-exp(-K*(data.age$Age-t02)))), data.age, 
                     start=list(K=0.1,t02=0), weights=data.age$N, lower=list(K=0.001,
                     t02=t0set), upper=list(K=2,t02=0.5), algorithm="port")
  t0          <- summary(model.vb)$parameters[2]
  SE.t0       <- summary(model.vb)$coefficients[2,2]
  K           <- summary(model.vb)$parameters[1]
  SE.K        <- summary(model.vb)$coefficients[1,2]
  SD.log.residuals.vb <- sd(resid(model.vb))
}

plot(data.age$Age, data.age$Length, main="Final VBGF", xlab="Age (years)",
     xlim=c(0,max(data.age.pr$Age)), ylim=c(0,max(1.1*Linf,max(data.age.pr$Length))), ylab="Length (cm)", pch=19, cex=0.5)
x=seq(0, max(data.age$Age), by=0.1)
y= Linf*(1-exp(-K*(x-t0)))
lines(x,y)

cat("-------------------------------------------------\n")
cat("Estimate of von Bertalanffy growth function\n")
cat("-------------------------------------------------\n")
cat("Number of observations =", sum(data.age$N), "\n")
cat("Observed maximum age =", max(data.age$Age), "years \n")
cat("Observed maximum length (including specimens without age)=", Lmax, "cm \n")
cat("Wetherall Linf =", WA.Linf, "cm, chosen Linf =", Linf, "\n")
cat("K =", K, ", 95% CL =", K-1.96*SE.K, "-", K+1.96*SE.K, "\n")
cat("t-zero =", t0, ", 95% CL =", t0-1.96*SE.t0, "-", t0+1.96*SE.t0, "(restricted to -3 and + 0.5 or Usert0)\n")
cat("SD of log(residuals) and of predicted log(length) =", SD.log.residuals.vb, "\n")
 } # end of inner VBGF loop
} # end of outer VBGF loop
  
# --------------------------------------------------------------------------------------------
# Analyse maturity data
# --------------------------------------------------------------------------------------------
if(length(data.org$Mat[data.org$Mat!=-9])>10) {
  m.okay <- TRUE
 # If cleaned LWR data with maturity info are available, use, else use data without weight
if(LWR.okay==TRUE && length(data.lw[data.lw$Mat != "-9",]) > 10) {
  # restrict data to those with maturity information for the chosen sex
  if(length(data.lw$Mat[data.lw$Mat=="1"]) > length(data.lw$Mat[data.lw$Mat=="61"])) {
    m.dat <- subset(data.lw, data.lw$Mat %in% c("1","2","3","4"), 
                 select=c("Length", "Mat", "N")) } else {
    m.dat <- subset(data.lw, data.lw$Mat %in% c("61","62","63","64"), 
                 select=c("Length", "Mat", "N"))  } } else {
  if(length(data.org$Mat[data.org$Mat=="1"]) > length(data.org$Mat[data.org$Mat=="61"])) {
     m.dat <- subset(data.org, data.org$Mat %in% c("1","2","3","4"), 
                 select=c("Length", "Mat", "N")) } else {
     m.dat <- subset(data.org, data.org$Mat %in% c("61","62","63","64"), 
                 select=c("Length", "Mat", "N"))  } }                                  

# determine available maturity stages
m.stages <- sort(unique(m.dat$Mat))
m.juv    <- min(m.stages)

cat("--------------------------------------------------------------","\n")
cat("Maturity analysis from proportion-mature-at-length data","\n")
cat("--------------------------------------------------------------","\n")
cat("Available maturity codes =", unique(m.dat$Mat),"\n")
cat("Number of observations =", sum(m.dat$N), "\n")
cat("Largest immature =", max(m.dat$Length[m.dat$Mat==m.juv]), "cm \n")
if(sum(m.dat$N) <=20) {
  cat("Immature lengths:", sort(m.dat$Length[m.dat$Mat==m.juv]),"\n")   }

if(length(m.stages) > 1 & length(m.dat$Mat) > 10) {
  m.okay <- TRUE} else {m.okay <- FALSE}

# continue maturity analysis if enough data are available
if(m.okay== TRUE) {
  # use counts in weighted histogram to determine proportion mature at length
  breaks <- 1:max(m.dat[,1]+1)  
  Lfreq.m.dat <- wtd.hist(m.dat[,1],breaks=breaks,weight=m.dat[,3],
                  plot=F)$counts 
  Lfreq.m.mature.dat <- wtd.hist(m.dat[m.dat[,2] !=m.juv, 1],
                  breaks=breaks, weight=m.dat[m.dat[,2] !=m.juv, 3],plot=F)$counts

  m.lengths.vector <- breaks[2:length(breaks)]+0.5
  m.mat.matrix <- cbind(Lfreq.m.mature.dat, Lfreq.m.dat - Lfreq.m.mature.dat) # create maturity matrix for glm
  m.mat.model <- glm(m.mat.matrix ~ m.lengths.vector, binomial) 
  m.Lm50 <- as.numeric((log(1/1) - m.mat.model$coefficients[1]) / m.mat.model$coefficients[2]) # gets the length at logit 1/1 = 0.5
  m.Lm10 <- as.numeric((log(1/9) - m.mat.model$coefficients[1]) / m.mat.model$coefficients[2]) # gets the length at logit 1/9 = 0.1
  m.Lm90 <- as.numeric((log(9/1) - m.mat.model$coefficients[1]) / m.mat.model$coefficients[2]) # gets the length at logit 9/1 = 0.9

  # continue display of results
  cat("Smallest mature =", min(m.dat$Length[m.dat$Mat != "61"]), "cm \n")
  if(m.Lm50 < 0.2 * Linf) {
  cat("Maturity data inappropriate for estimating length at 50% maturity \n") 
  } else {
  cat("Ogive length at 50% maturity =", m.Lm50, "cm \n") }
  cat("Ogive length at 10% and 90% maturity", m.Lm10, " -", m.Lm90, "cm \n")

  #------------------------------------
  # plot maturity data
  #------------------------------------
  windows(12,6)
  par(mfrow=c(1,2))
  # plot mature-at-length ogives
  m.prop.mat <-Lfreq.m.mature.dat / Lfreq.m.dat
  plot(x=m.lengths.vector,y=m.prop.mat, xlim=c(0,ceiling(max(L))), ylim=c(0,1), xlab="Length (cm)", 
     ylab="Proportion mature ", main="Proportion mature-at-length", frame=F)
  lines(m.lengths.vector,predict(m.mat.model,list(density=m.lengths.vector),type="response"))
  points(x=c(m.Lm10, m.Lm50, m.Lm90), y = c(0.1, 0.5, 0.9), col="red", cex=2)
  text(x=m.Lm50, y=0.5, adj=c(-0.3,0.1), "Lm50")
  text(x=m.Lm10, y=0.1, adj=c(-0.3,0.1), "Lm10")
  text(x=m.Lm90, y=0.9, adj=c(-0.3,0.1), "Lm90")	

  #---------------------------------------------------------------
  # Analyze proportion mature at age
  # -----------------------------------------------------------------
  if(length(data.age$Mat[data.age$Mat %in% m.stages]) > 5) {
  m.maturity.age <- subset(data.age[data.age$Mat %in% m.stages,], select=c("Age", "Mat", "N")) # data needed to determine proportion mature per age
  Lfreq.m.maturity.age <- wtd.hist(m.maturity.age[,1],breaks=1:(max(m.maturity.age[,1])+1), 
	  	weight=m.maturity.age[,3],plot=F)$counts # use counts in weighted histogram
  Lfreq.m.mature.age <- wtd.hist(m.maturity.age[m.maturity.age[,2] !=m.juv, 1],
	  	breaks=1:(max(m.maturity.age[,1])+1), weight=m.maturity.age[m.maturity.age[,2] !=m.juv, 3],plot=F)$counts
  m.mat.age <-  Lfreq.m.mature.age / Lfreq.m.maturity.age  # proportion mature at age 
  }
  cat("-----------------------------------------------------------","\n")
  cat("Maturity analysis from proportion-mature-at-age data","\n")
  cat("-----------------------------------------------------------","\n")
  cat("Number of observations =", sum(m.maturity.age$N), "\n")
  cat("Youngest mature =", min(m.maturity.age$Age[m.maturity.age$Mat !=m.juv]), "years \n")
  cat("Oldest immature =", max(m.maturity.age$Age[m.maturity.age$Mat==m.juv]), "years \n")
  cat("Proportion mature at ages 1-6 =", format(m.mat.age[1:6],digits=1), "\n")

  # create a plot of mature-at-age ogive
  barplot(m.mat.age[1:6], names.arg=c(1,2,3,4,5,6), ylim=c(0,1), main="Proportion mature-at-age", xlab="Age", col="white")
  } # end of m.okay (=enough age at maturity data) loop
} # end of maturity loop

cat("---------------------------------------------\n")
cat("Other LF results for this survey gear        \n")
cat("---------------------------------------------\n")
cat("Fully selected length (peak+1) Lv =", Lv, "cm\n")
cat("Length at first capture        Lc =", Lc, "cm\n")
cat("Weighted mean length        Lmean =", Lmean,"\n")
if(is.na(M)==F & is.na(K)==F & is.na(FM)==F) {
  cat("Length at max cohort biomass Lopt =", Linf*3/(3+M/K), "\n") 
  cat("Lc resulting in Lopt       Lc_opt =", Linf*(2+3*FM/M)/((1+FM/M)*(3+M/K)), "\n")
  cat("Mean survey length if F=M  L(F=M) =", (K*Linf+2*Lc*M)/(K+2*M),"cm \n")
} else {
  cat("Length at max cohort biomass Lopt =", 2/3*Linf,"# assuming b ~ 3 and M/K ~ 1.5 \n")
  cat("Lc resulting in Lopt       Lc_opt =", 0.5*Linf,", if F ~ M \n")
  cat("Lc resulting in Lopt       Lc_opt =", 0.55*Linf,", if F ~ 2 M \n")
  cat("Mean survey length if F=M  L(F=M) =", 0.75*Lc+0.25*Linf,"\n")
}
cat("---------------------------------------------\n")
cat("External estimates of mortality        \n")
cat("---------------------------------------------\n")
cat("Natural mortality of adults     M =", M, "1/y \n")
cat("Commercial fishing mortality    F =", FM, "1/y \n\n")
cat("Comment:", Comment, "\n")

cat("\n Summary, formatted for pasting into other R-code \n")
cat(" ------------------------------------------------ \n")
cat(" Species   <-", Species, "\n")
cat(" Sex       <-", Sex, "\n")
cat(" Area      <- c(", sort(unique(data.raw$Area)),") \n")
if(length(data.org$Weight[is.na(data.org$Weight)==F &data.org$Weight!= -9]) >=10) {
  cat(" a         <-", 10^log10a, "\n")
  cat(" b         <-", b, "\n")}
Linf  <- ifelse(is.na(UserLinf)==T,WA.Linf,UserLinf)
cat(" Linf      <-",Linf ,"\n")
if(is.na(K)==F) {
  cat(" K         <-", K,"\n")
  cat(" t0        <-", format(t0,digits=2),"\n") }
if(m.okay==TRUE) {
  cat(" Lm50      <-", m.Lm50, "\n")
  cat(" Lm90      <-", m.Lm90, "\n")
}
cat(" M         <-", M,"\n")
cat(" FM        <-", FM,"\n")
if(is.na(M)==F & is.na(K)==F & is.na(FM)==F) {
  cat(" Lopt      <-", Linf*3/(3+M/K), "\n") 
  cat(" Lc_opt    <-", Linf*(2+3*FM/M)/((1+FM/M)*(3+M/K)), "\n")
} else {
  cat(" Lopt       <-", 2/3*Linf,"# assuming b ~ 3 and M/K ~ 1.5 \n")
  cat(" Lc_opt     <-", 0.5*Linf,"# if F ~ M \n")
  cat(" Lc_opt     <-", 0.55*Linf,"# if F ~ 2 M \n")
}